// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO12/16AO12_Linux_3.x.x.x.x_GSC_DN/driver/16ao12.h $
// $Rev: 52799 $
// $Date: 2023-04-06 16:33:55 -0500 (Thu, 06 Apr 2023) $

// 16AO12: Device Driver: header file

#ifndef	__16AO12_H__
#define	__16AO12_H__

#include "gsc_common.h"
#include "gsc_pci9080.h"



// macros *********************************************************************

#define	AO12_BASE_NAME						"16ao12"

// IOCTL command codes
#define	AO12_IOCTL_REG_READ					OS_IOCTL_RW( 0, 12, gsc_reg_t)
#define	AO12_IOCTL_REG_WRITE				OS_IOCTL_W ( 1, 12, gsc_reg_t)
#define	AO12_IOCTL_REG_MOD					OS_IOCTL_W ( 2, 12, gsc_reg_t)
#define	AO12_IOCTL_QUERY					OS_IOCTL_RW( 3,  4, s32)
#define	AO12_IOCTL_INITIALIZE				OS_IOCTL   ( 4)
#define	AO12_IOCTL_AUTO_CALIBRATE			OS_IOCTL   ( 5)
#define	AO12_IOCTL_AUTO_CAL_STS				OS_IOCTL_R ( 6,  4, s32)
#define	AO12_IOCTL_BUFFER_CLEAR				OS_IOCTL   ( 7)
#define	AO12_IOCTL_BUFFER_MODE				OS_IOCTL_RW( 8,  4, s32)
#define	AO12_IOCTL_BUFFER_SIZE				OS_IOCTL_RW( 9,  4, s32)
#define	AO12_IOCTL_BUFFER_STATUS			OS_IOCTL_R (10,  4, s32)
#define	AO12_IOCTL_BURST_ENABLE				OS_IOCTL_RW(11,  4, s32)
#define	AO12_IOCTL_BURST_READY				OS_IOCTL_R (12,  4, s32)
#define	AO12_IOCTL_BURST_TRIGGER			OS_IOCTL   (13)
#define	AO12_IOCTL_CHANNEL_SEL				OS_IOCTL_RW(14,  4, s32)
#define	AO12_IOCTL_CLOCK_ENABLE				OS_IOCTL_RW(15,  4, s32)
#define	AO12_IOCTL_CLOCK_READY				OS_IOCTL_R (16,  4, s32)
#define	AO12_IOCTL_CLOCK_REF_SRC			OS_IOCTL_RW(17,  4, s32)
#define	AO12_IOCTL_CLOCK_SRC				OS_IOCTL_RW(18,  4, s32)
#define	AO12_IOCTL_CLOCK_SW					OS_IOCTL   (19)
#define	AO12_IOCTL_DATA_FORMAT				OS_IOCTL_RW(20,  4, s32)
#define	AO12_IOCTL_GROUND_SENSE				OS_IOCTL_RW(21,  4, s32)
#define	AO12_IOCTL_IRQ_SEL					OS_IOCTL_RW(22,  4, s32)
#define	AO12_IOCTL_LOAD_READY				OS_IOCTL_R (23,  4, s32)
#define	AO12_IOCTL_LOAD_REQUEST				OS_IOCTL   (24)
#define	AO12_IOCTL_NCLK						OS_IOCTL_RW(25,  4, s32)
#define	AO12_IOCTL_NRATE					OS_IOCTL_RW(26,  4, s32)
#define	AO12_IOCTL_OUTPUT_MODE				OS_IOCTL_RW(27,  4, s32)
#define	AO12_IOCTL_TX_IO_ABORT				OS_IOCTL_R (28,  4, s32)
#define	AO12_IOCTL_TX_IO_MODE				OS_IOCTL_RW(29,  4, s32)
#define	AO12_IOCTL_TX_IO_TIMEOUT			OS_IOCTL_RW(30,  4, s32)
#define	AO12_IOCTL_WAIT_CANCEL				OS_IOCTL_RW(31, 28, gsc_wait_t)
#define	AO12_IOCTL_WAIT_EVENT				OS_IOCTL_RW(32, 28, gsc_wait_t)
#define	AO12_IOCTL_WAIT_STATUS				OS_IOCTL_RW(33, 28, gsc_wait_t)



//*****************************************************************************
// AO12_IOCTL_REG_READ
// AO12_IOCTL_REG_WRITE
// AO12_IOCTL_REG_MOD
//
#define	AO12_REG_ENCODE(s,o)				GSC_REG_ENCODE(GSC_REG_TYPE_BAR2,(s),(o))
// Parameter:	gsc_reg_t*
#define	AO12_GSC_BCR						AO12_REG_ENCODE(4,0x00)// Board Control Register
#define	AO12_GSC_CSR						AO12_REG_ENCODE(4,0x04)// Channel Selection Register
#define	AO12_GSC_SRR						AO12_REG_ENCODE(4,0x08)// Sample Rate Register
#define	AO12_GSC_BOR						AO12_REG_ENCODE(4,0x0C)// Buffer Operations Register
#define	AO12_GSC_ODBR						AO12_REG_ENCODE(4,0x18)// Output Data Buffer Register
#define	AO12_GSC_ACR						AO12_REG_ENCODE(4,0x1C)// Adjustable Clock Register

//*****************************************************************************
// AO12_IOCTL_QUERY
//
//	Parameter:	s32
//		Pass in a value from the list below.
//		The value returned is the answer to the query.

typedef enum
{
	AO12_QUERY_AUTO_CAL_MS,		// Max auto-cal period in ms.
	AO12_QUERY_CHANNEL_MAX,		// Maximum number of channels supported.
	AO12_QUERY_CHANNEL_QTY,		// The number of channels installed.
	AO12_QUERY_COUNT,			// Number of query options.
	AO12_QUERY_DEVICE_TYPE,		// Value from gsc_dev_type_t
	AO12_QUERY_FREF_ALT_MAX,	// The alternate maximum Fref.
	AO12_QUERY_FREF_ALT_MIN,	// The alternate minimum Fref.
	AO12_QUERY_FREF_DEFAULT,	// The default Fref value for the device.
	AO12_QUERY_FIFO_SIZE,		// The FIFO size in 32-bit samples.
	AO12_QUERY_FSAMP_MAX,		// Rate Generator maximum output rate.
	AO12_QUERY_FSAMP_MIN,		// Rate Generator minimum output rate.
	AO12_QUERY_INIT_MS,			// Max initialize period in ms.
	AO12_QUERY_NCLK_MAX,		// Maximum rate divider Nclk value.
	AO12_QUERY_NCLK_MIN,		// Maximum rate divider Nclk value.
	AO12_QUERY_NRATE_MAX,		// Maximum rate generator Nrate value.
	AO12_QUERY_NRATE_MIN,		// Maximum rate generator Nrate value.
	AO12_QUERY_LAST				// Must be very last entry.

} ao12_query_t;

#define	AO12_IOCTL_QUERY_ERROR	(-1)	// A query request is unknown of invalid.

//*****************************************************************************
// AO12_IOCTL_INITIALIZE					BCR D15
//
//	Parameter:	None

//*****************************************************************************
// AO12_IOCTL_AUTO_CALIBRATE				BCR D12-D13
//
//	Parameter:	None

//*****************************************************************************
// AO12_IOCTL_AUTO_CAL_STS					BCR D12-D14
//
//	Parameter:	s32*
//		One of the below values is returned.
#define	AO12_AUTO_CAL_STS_ACTIVE			0
#define	AO12_AUTO_CAL_STS_FAIL				1
#define	AO12_AUTO_CAL_STS_PASS				2

//*****************************************************************************
// AO12_IOCTL_BUFFER_CLEAR					BOR D11
//
//	Parameter:	None.

//*****************************************************************************
// AO12_IOCTL_BUFFER_MODE					BOR D8
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO12_BUFFER_MODE_OPEN				0
#define	AO12_BUFFER_MODE_CIRC				1

//*****************************************************************************
// AO12_IOCTL_BUFFER_SIZE					BOR D0-D3
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO12_BUFFER_SIZE_4					0
#define	AO12_BUFFER_SIZE_8					1
#define	AO12_BUFFER_SIZE_16					2
#define	AO12_BUFFER_SIZE_32					3
#define	AO12_BUFFER_SIZE_64					4
#define	AO12_BUFFER_SIZE_128				5
#define	AO12_BUFFER_SIZE_256				6
#define	AO12_BUFFER_SIZE_512				7
#define	AO12_BUFFER_SIZE_1K					8
#define	AO12_BUFFER_SIZE_2K					9
#define	AO12_BUFFER_SIZE_4K					10
#define	AO12_BUFFER_SIZE_8K					11
#define	AO12_BUFFER_SIZE_16K				12
#define	AO12_BUFFER_SIZE_32K				13
#define	AO12_BUFFER_SIZE_64K				14
#define	AO12_BUFFER_SIZE_128K				15

//*****************************************************************************
// AO12_IOCTL_BUFFER_STATUS					BOR D12-D15
//
//	Parameter:	s32
//		The service returns one of the below listed values.
#define	AO12_BUFFER_STATUS_EMPTY			0
#define	AO12_BUFFER_STATUS_1Q_FULL			1
#define	AO12_BUFFER_STATUS_MEDIUM			2
#define	AO12_BUFFER_STATUS_3Q_FULL			3
#define	AO12_BUFFER_STATUS_FULL				4

//*****************************************************************************
// AO12_IOCTL_BURST_ENABLE					BCR D0
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO12_BURST_ENABLE_NO				0
#define	AO12_BURST_ENABLE_YES				1

//*****************************************************************************
// AO12_IOCTL_BURST_READY					BCR D1
//
//	Parameter:	s32
//		The service returns one of the below listed values.
#define	AO12_BURST_READY_NO					0
#define	AO12_BURST_READY_YES				1

//*****************************************************************************
// AO12_IOCTL_BURST_TRIGGER					BCR D2
//
//	Parameter:	None.

//*****************************************************************************
// AO12_IOCTL_CHANNEL_SEL					CSR D0-D11
//
//	Parameter:	s32
//		Pass in any valid bitwise combination from 0x0000 to 0xFFF, or
//		-1 to read the current setting.

//*****************************************************************************
// AO12_IOCTL_CLOCK_ENABLE					BOR D5
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO12_CLOCK_ENABLE_NO				0
#define	AO12_CLOCK_ENABLE_YES				1

//*****************************************************************************
// AO12_IOCTL_CLOCK_READY					BOR D6
//
//	Parameter:	s32
//		The service returns one of the below listed values.
#define	AO12_CLOCK_READY_NO					0
#define	AO12_CLOCK_READY_YES				1

//*****************************************************************************
// AO12_IOCTL_CLOCK_REF_SRC					ACR D9
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO12_CLOCK_REF_SRC_PRI				0
#define	AO12_CLOCK_REF_SRC_ALT				1

//*****************************************************************************
// AO12_IOCTL_CLOCK_SRC						BOR D4
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO12_CLOCK_SRC_INT					0
#define	AO12_CLOCK_SRC_EXT_SW				1

//*****************************************************************************
// AO12_IOCTL_CLOCK_SW						BOR D7
//
//	Parameter:	None.

//*****************************************************************************
// AO12_IOCTL_DATA_FORMAT					BCR D4
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO12_DATA_FORMAT_2S_COMP			0	// Twos Compliment
#define	AO12_DATA_FORMAT_OFF_BIN			1	// Offset Binary

//*****************************************************************************
// AO12_IOCTL_GROUND_SENSE					BCR D3
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO12_GROUND_SENSE_DISABLE			0
#define	AO12_GROUND_SENSE_REMOTE			1

//*****************************************************************************
// AO12_IOCTL_IRQ_SEL						BCR D8-D10
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO12_IRQ_INIT_DONE					0
#define	AO12_IRQ_AUTO_CAL_DONE				1
#define	AO12_IRQ_BUF_EMPTY					2
#define	AO12_IRQ_BUF_1Q_FULL				3
#define	AO12_IRQ_BUF_3Q_FULL				4
#define	AO12_IRQ_BURST_TRIG_READY			5
#define	AO12_IRQ_LOAD_READY					6
#define	AO12_IRQ_LOAD_READY_END				7

//*****************************************************************************
// AO12_IOCTL_LOAD_READY					BOR D10
//
//	Parameter:	s32
//		The service returns one of the below listed values.
#define	AO12_LOAD_READY_NO					0
#define	AO12_LOAD_READY_YES					1

//*****************************************************************************
// AO12_IOCTL_LOAD_REQUEST					BOR D9
//	The driver does not wait for the ready state to appear.
//
//	Parameter:	None

//*****************************************************************************
// AO12_IOCTL_NCLK							ACR D0-D8
//
//	Parameter:	s32
//		Pass in any valid value, or
//		-1 to read the current setting.
//		The valid range is 0 to 0x1FF.

//*****************************************************************************
// AO12_IOCTL_NRATE							SRR D0-D15
//
//	Parameter:	s32
//		Pass in any valid value, or
//		-1 to read the current setting.
//		The valid range is 75 to 0xFFFF.

//*****************************************************************************
// AO12_IOCTL_OUTPUT_MODE					BCR D7
//
//	Parameter:	s32
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	AO12_OUTPUT_MODE_SEQ				0
#define	AO12_OUTPUT_MODE_SIM				1

//*****************************************************************************
// AO12_IOCTL_TX_IO_ABORT
//
//	Parameter:	s32*
//		The returned value is one of the below options.
#define	AO12_IO_ABORT_NO					0
#define	AO12_IO_ABORT_YES					1

//*****************************************************************************
// AO12_IOCTL_TX_IO_MODE
//
//	Parameter:	s32
//		Pass in any of the gsc_io_mode_t options, or
//		-1 to read the current setting.
#define	AO12_IO_MODE_DEFAULT				GSC_IO_MODE_PIO
// GSC_IO_MODE_PIO
// GSC_IO_MODE_BMDMA

//*****************************************************************************
// AO12_IOCTL_TX_IO_TIMEOUT (in seconds)
//
//	Parameter:	s32
//		Pass in any value from the minimim to the maximim, the infinite option,
//		or -1 to read the current setting. The value -1 is returned if the
//		feature is not supported, but this should never happen.
#define	AO12_IO_TIMEOUT_DEFAULT				10
#define	AO12_IO_TIMEOUT_NO_SLEEP			0		// Do not sleep to wait for more data.
#define	AO12_IO_TIMEOUT_MIN					0
#define	AO12_IO_TIMEOUT_MAX					GSC_IO_TIMEOUT_MAX
#define	AO12_IO_TIMEOUT_INFINITE			GSC_IO_TIMEOUT_INFINITE

//*****************************************************************************
// AO12_IOCTL_WAIT_EVENT					all fields must be valid
// AO12_IOCTL_WAIT_CANCEL					fields need not be valid
// AO12_IOCTL_WAIT_STATUS					fields need not be valid
//
//	Parameter:	gsc_wait_t*
// gsc_wait_t.flags - see gsc_common.h
// gsc_wait_t.main - see gsc_common.h
// gsc_wait_t.gsc
#define	AO12_WAIT_GSC_INIT_DONE				0x0001
#define	AO12_WAIT_GSC_AUTO_CAL_DONE			0x0002
#define	AO12_WAIT_GSC_BUF_EMPTY				0x0004
#define	AO12_WAIT_GSC_BUF_1Q_FULL			0x0008
#define	AO12_WAIT_GSC_BUF_3Q_FULL			0x0010
#define	AO12_WAIT_GSC_BURST_TRIG_RDY		0x0020
#define	AO12_WAIT_GSC_LOAD_READY			0x0040
#define	AO12_WAIT_GSC_LOAD_READY_END		0x0080
#define	AO12_WAIT_GSC_ALL					0x00FF
// gsc_wait_t.alt flags
#define	AO12_WAIT_ALT_ALL					0x0000
// gsc_wait_t.io
#define	AO12_WAIT_IO_TX_ABORT				0x0001
#define	AO12_WAIT_IO_TX_DONE				0x0002
#define	AO12_WAIT_IO_TX_ERROR				0x0004
#define	AO12_WAIT_IO_TX_TIMEOUT				0x0008
#define	AO12_WAIT_IO_ALL					0x000F



#endif
