// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO12/16AO12_Linux_3.x.x.x.x_GSC_DN/driver/main.h $
// $Rev: 52799 $
// $Date: 2023-04-06 16:33:55 -0500 (Thu, 06 Apr 2023) $

// 16AO12: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "16ao12.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_WAIT
#define	DEV_SUPPORTS_WRITE
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"16AO12"	// Upper case form of the below.
#define	DEV_NAME							"16ao12"	// MUST AGREE WITH AO12_BASE_NAME

#define	DEV_VERSION							"4.2"		// FOR DEVICE SPECIFIC CODE ONLY!
// 4.2	Updated to support the 6.x series kernel.
//		Updated comments for consistency.
//		Updated macros for consistency.
//		Updated the Initialize IOCTL service for consistency.
//		Updated the Autocalibration service for consistency.
//		Enhanced the Buffer Clear IOCTL service for consistency.
//		Updated per PLX code segregation.
// 4.1	Added support for the 5.x kernel series.
//		Removed compiler warnings for Fedora 31.
//		Updated for porting to Windows.
//		Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
//		LINTed sources.
//		Updated comments for consistency.
// 4.0	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Modified code so open succeeds even if initialization fails.
//		Renamed most 16AO12 specific macros for consistency with other drivers.
// 3.0	Overhauled driver.
// 2.3	Updated for greater kernel support.
//		Removed lots of dead code.
//		Ported to use common proc code.
//		Updated build files to use more common code.
// 2.2	Added the version number from the gsc_common sources.
// 2.1	Updated for the 3.x kernel.
// 2.0	Ported to Fedora 15.
//		Sized BAR regions per their size rather than hard code.
//		The /proc entry is deleted only after it is created.
//		Removed "gsc" prefix from device name.
//		Made correction for 2.4 and 2.2 kernel support.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				0
#define	DEV_IO_TX_STREAM_QTY				1

#define	GSC_WRITE_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					AO12_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					AO12_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						AO12_WAIT_IO_ALL

#define	BUFFER_EMPTY						(1L << 12)
#define	BUFFER_FULL_1Q						(1L << 13)
#define	BUFFER_FULL_3Q						(1L << 14)
#define	BUFFER_FULL							(1L << 15)



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					io_mode;			// PIO, DMDMA
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.

	// Initialized by I/O service at time of use.

	int					abort;
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	os_pci_t			pci;				// The kernel PCI device descriptor.
	os_data_t			os;					// OS specific data.
	os_spinlock_t		spinlock;			// Control ISR access.
	os_sem_t			sem;				// Control thread access.
	gsc_dev_type_t		board_type;			// Corresponds to basic device type.
	const char*			model;				// Base model number in upper case.
	int					board_index;		// Index of the device being accessed.
	int					users;				// Number of currently active open() requests.
	int					share;				// Were we opened in shared mode?

	gsc_bar_t			bar;				// device register mappings
	gsc_dma_t			dma;				// For DMA based I/O.
	gsc_irq_t			irq;				// For interrut support.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		tx;				// Analog Output write
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;		// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;		// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;		// DMA Threshold Register

		VADDR_T			gsc_bcr_32;			// 0x00 Board Control Register
		VADDR_T			gsc_csr_32;			// 0x04 Channel Selection Register
		VADDR_T			gsc_srr_32;			// 0x08 Sample Rate Register
		VADDR_T			gsc_bor_32;			// 0x0C Buffer Operations Register

		VADDR_T			gsc_odbr_32;		// 0x18 Output Data Buffer Register
		VADDR_T			gsc_acr_32;			// 0x1C Adjustable Clock Register

	} vaddr;

	struct
	{
		int				auto_cal_ms;		// Maximum ms for auto-cal
		int				initialize_ms;		// Maximum ms for initialize

		s32				channel_mask;		// The channel selection mask.
		s32				channel_max;		// Maximum channels supported by model.
		s32				channel_qty;		// The number of channels installed.

		s32				fifo_size;			// FIFO size in 32-bit samples.
		s32				fref_alt_max;		// The alternate maximum Fref.
		s32				fref_alt_min;		// The alternate minimum Fref.
		s32				fref_default;		// Default reference clock rate.
		s32				fsamp_max;			// Maximum sample rate.
		s32				fsamp_min;			// Minimum sample rate.

		s32				nclk_max;			// Minimum Nclk value.
		s32				nclk_min;			// Maximum Nclk value.
		s32				nrate_max;			// Minimum Nrate value.
		s32				nrate_min;			// Maximum Nrate value.

	} cache;
};



// prototypes *****************************************************************

int			buffer_size(dev_data_t* dev, u32* size);

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_write_select(dev_data_t* dev, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_write_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
