// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO12/16AO12_Linux_3.x.x.x.x_GSC_DN/utils/util_auto_cal_sts.c $
// $Rev: 52792 $
// $Date: 2023-04-06 16:25:59 -0500 (Thu, 06 Apr 2023) $

// 16AO12: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao12_auto_cal_sts
*
*	Purpose:
*
*		Provide a visual wrapper for the AO12_IOCTL_AUTO_CAL_STS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao12_auto_cal_sts(int fd, int index, int verbose, s32* get)
{
	char	buf[128];
	int		errs;
	char*	ptr;
	int		ret;
	s32		set;

	if (verbose)
		gsc_label_index("Auto-Cal Status", index);

	ret		= ao12_ioctl(fd, AO12_IOCTL_AUTO_CAL_STS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case AO12_AUTO_CAL_STS_ACTIVE:

			ptr	= "Active";
			break;

		case AO12_AUTO_CAL_STS_FAIL:

			ptr	= "Fail";
			break;

		case AO12_AUTO_CAL_STS_PASS:

			ptr	= "Pass";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


