// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO12/16AO12_Linux_3.x.x.x.x_GSC_DN/utils/util_buffer_size.c $
// $Rev: 52792 $
// $Date: 2023-04-06 16:25:59 -0500 (Thu, 06 Apr 2023) $

// 16AO12: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao12_buffer_size
*
*	Purpose:
*
*		Provide a visual wrapper for the AO12_IOCTL_BUFFER_SIZE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao12_buffer_size(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	char*	ptr;
	int		ret;

	if (verbose)
		gsc_label_index("Buffer Size", index);

	ret		= ao12_ioctl(fd, AO12_IOCTL_BUFFER_SIZE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case AO12_BUFFER_SIZE_4:

			ptr	= "4";
			break;

		case AO12_BUFFER_SIZE_8:

			ptr	= "8";
			break;

		case AO12_BUFFER_SIZE_16:

			ptr	= "16";
			break;

		case AO12_BUFFER_SIZE_32:

			ptr	= "32";
			break;

		case AO12_BUFFER_SIZE_64:

			ptr	= "64";
			break;

		case AO12_BUFFER_SIZE_128:

			ptr	= "128";
			break;

		case AO12_BUFFER_SIZE_256:

			ptr	= "256";
			break;

		case AO12_BUFFER_SIZE_512:

			ptr	= "512";
			break;

		case AO12_BUFFER_SIZE_1K:

			ptr	= "1,024";
			break;

		case AO12_BUFFER_SIZE_2K:

			ptr	= "2,048";
			break;

		case AO12_BUFFER_SIZE_4K:

			ptr	= "4,096";
			break;

		case AO12_BUFFER_SIZE_8K:

			ptr	= "8,192";
			break;

		case AO12_BUFFER_SIZE_16K:

			ptr	= "16,384";
			break;

		case AO12_BUFFER_SIZE_32K:

			ptr	= "32,768";
			break;

		case AO12_BUFFER_SIZE_64K:

			ptr	= "65,536";
			break;

		case AO12_BUFFER_SIZE_128K:

			ptr	= "131,072";
			break;
	}

	if (verbose)
		printf("%s  (%s samples deep)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


