// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO12/16AO12_Linux_3.x.x.x.x_GSC_DN/utils/util_buffer_status.c $
// $Rev: 52792 $
// $Date: 2023-04-06 16:25:59 -0500 (Thu, 06 Apr 2023) $

// 16AO12: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao12_buffer_status
*
*	Purpose:
*
*		Provide a visual wrapper for the AO12_IOCTL_BUFFER_STATUS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao12_buffer_status(int fd, int index, int verbose, s32* get)
{
	char	buf[128];
	int		errs;
	char*	ptr;
	int		ret;
	s32		set;

	if (verbose)
		gsc_label_index("Buffer Status", index);

	ret		= ao12_ioctl(fd, AO12_IOCTL_BUFFER_STATUS, &set);
	errs	= ret ? 1 : 0;

	switch (get[0])
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "INVALID: %ld", (long) get[0]);
			break;

		case AO12_BUFFER_STATUS_EMPTY:

			ptr	= "Empty";
			break;

		case AO12_BUFFER_STATUS_1Q_FULL:

			ptr	= "< 1/4 Full";
			break;

		case AO12_BUFFER_STATUS_MEDIUM:

			ptr	= "Medium";
			break;

		case AO12_BUFFER_STATUS_3Q_FULL:

			ptr	= ">= 3/4 Full";
			break;

		case AO12_BUFFER_STATUS_FULL:

			ptr	= "Full";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


