// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO12/16AO12_Linux_3.x.x.x.x_GSC_DN/utils/util_config_ao.c $
// $Rev: 52792 $
// $Date: 2023-04-06 16:25:59 -0500 (Thu, 06 Apr 2023) $

// 16AO12: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao12_config_ao
*
*	Purpose:
*
*		Configure the basic services.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the deice to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	The desired sample rate.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	ao12_config_ao(int fd, int index, int verbose, s32 fsamp)
{
	int	errs	= 0;
	s32	nclk;
	s32	nrate;
	s32	ref_src;

	if (verbose)
	{
		gsc_label_index("Output Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= ao12_initialize			(fd, index, verbose);
	errs	+= ao12_auto_calibrate		(fd, index, verbose);

	errs	+= ao12_tx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,					NULL);
	errs	+= ao12_tx_io_timeout		(fd, index, verbose, AO12_IO_TIMEOUT_DEFAULT,			NULL);

	errs	+= ao12_buffer_mode			(fd, index, verbose, AO12_BUFFER_MODE_OPEN,				NULL);
	errs	+= ao12_buffer_size			(fd, index, verbose, AO12_BUFFER_SIZE_128K,				NULL);
	errs	+= ao12_burst_enable		(fd, index, verbose, AO12_BURST_ENABLE_NO,				NULL);
	errs	+= ao12_channel_sel			(fd, index, verbose, 0xFFF,								NULL);
	errs	+= ao12_output_mode			(fd, index, verbose, AO12_OUTPUT_MODE_SIM,				NULL);
	errs	+= ao12_data_format			(fd, index, verbose, AO12_DATA_FORMAT_OFF_BIN,			NULL);
	errs	+= ao12_ground_sense		(fd, index, verbose, AO12_GROUND_SENSE_DISABLE,			NULL);
	errs	+= ao12_fsamp_ao_compute	(fd, index, verbose, fsamp, &ref_src, &nrate, &nclk,	NULL);
	errs	+= ao12_clock_src			(fd, index, verbose, AO12_CLOCK_SRC_INT,				NULL);
	errs	+= ao12_clock_ref_src		(fd, index, verbose, ref_src,							NULL);
	errs	+= ao12_nrate				(fd, index, verbose, nrate,								NULL);
	errs	+= ao12_nclk				(fd, index, verbose, nclk,								NULL);
	errs	+= ao12_clock_enable		(fd, index, verbose, AO12_CLOCK_ENABLE_YES,				NULL);

	errs	+= ao12_fsamp_report_all	(fd, index, verbose,									NULL);

	errs	+= ao12_buffer_clear		(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


