// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO12/16AO12_Linux_3.x.x.x.x_GSC_DN/utils/util_irq_sel.c $
// $Rev: 52792 $
// $Date: 2023-04-06 16:25:59 -0500 (Thu, 06 Apr 2023) $

// 16AO12: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao12_irq_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the AO12_IOCTL_IRQ_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao12_irq_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	char*	ptr;
	int		ret;

	if (verbose)
		gsc_label_index("IRQ Selection", index);

	ret		= ao12_ioctl(fd, AO12_IOCTL_IRQ_SEL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case AO12_IRQ_INIT_DONE:

			ptr	= "Initialization Done";
			break;

		case AO12_IRQ_AUTO_CAL_DONE:

			ptr	= "Auto-Calibration Done";
			break;

		case AO12_IRQ_BUF_EMPTY:

			ptr	= "Buffer Empty";
			break;

		case AO12_IRQ_BUF_1Q_FULL:

			ptr	= "Buffer <= 1/4 Full";
			break;

		case AO12_IRQ_BUF_3Q_FULL:

			ptr	= "Buffer > 1/4 Full";
			break;

		case AO12_IRQ_BURST_TRIG_READY:

			ptr	= "Burst Trigger Ready";
			break;

		case AO12_IRQ_LOAD_READY:

			ptr	= "Load Ready";
			break;

		case AO12_IRQ_LOAD_READY_END:

			ptr	= "Load Ready End";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


