// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO12/16AO12_Linux_3.x.x.x.x_GSC_DN/utils/util_query.c $
// $Rev: 52792 $
// $Date: 2023-04-06 16:25:59 -0500 (Thu, 06 Apr 2023) $

// 16AO12: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao12_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AO12_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao12_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AO12_QUERY_AUTO_CAL_MS:		ptr	= "Auto-Cal Duration";	break;
		case AO12_QUERY_CHANNEL_MAX:		ptr	= "Max Channels";		break;
		case AO12_QUERY_CHANNEL_QTY:		ptr	= "Installed Channels";	break;
		case AO12_QUERY_COUNT:				ptr	= "Query Options";		break;
		case AO12_QUERY_DEVICE_TYPE:		ptr	= "Device Type";		break;
		case AO12_QUERY_FIFO_SIZE:			ptr	= "FIFO Size";			break;
		case AO12_QUERY_FSAMP_MAX:			ptr	= "Fsamp Max";			break;
		case AO12_QUERY_FSAMP_MIN:			ptr	= "Fsamp Min";			break;
		case AO12_QUERY_INIT_MS:			ptr	= "Init. Duration";		break;
		case AO12_QUERY_NCLK_MAX:			ptr	= "Nclk Max";			break;
		case AO12_QUERY_NCLK_MIN:			ptr	= "Nclk Min";			break;
		case AO12_QUERY_NRATE_MAX:			ptr	= "Nrate Max";			break;
		case AO12_QUERY_NRATE_MIN:			ptr	= "Nrate Min";			break;
		case AO12_QUERY_FREF_ALT_MAX:		ptr	= "Fref Default Max";	break;
		case AO12_QUERY_FREF_ALT_MIN:		ptr	= "Fref Default Min";	break;
		case AO12_QUERY_FREF_DEFAULT:		ptr	= "Fref Default";		break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= ao12_ioctl(fd, AO12_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AO12_QUERY_CHANNEL_MAX:
		case AO12_QUERY_CHANNEL_QTY:
		case AO12_QUERY_COUNT:
		case AO12_QUERY_NCLK_MAX:
		case AO12_QUERY_NCLK_MIN:
		case AO12_QUERY_NRATE_MAX:
		case AO12_QUERY_NRATE_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case AO12_QUERY_FREF_ALT_MAX:
		case AO12_QUERY_FREF_ALT_MIN:
		case AO12_QUERY_FREF_DEFAULT:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case AO12_QUERY_AUTO_CAL_MS:
		case AO12_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case AO12_QUERY_FSAMP_MAX:
		case AO12_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case AO12_QUERY_FIFO_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case AO12_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_16AO12)
			{
				strcpy(buf, "16AO12");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


