// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO12/16AO12_Linux_3.x.x.x.x_GSC_DN/utils/util_tx_io_mode.c $
// $Rev: 52792 $
// $Date: 2023-04-06 16:25:59 -0500 (Thu, 06 Apr 2023) $

// 16AO12: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao12_tx_io_mode
*
*	Purpose:
*
*		Provide a visual wrapper for the AO12_IOCTL_TX_IO_MODE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao12_tx_io_mode(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	char*	ptr;
	int		ret;

	if (verbose)
		gsc_label_index("Tx I/O Mode", index);

	ret		= ao12_ioctl(fd, AO12_IOCTL_TX_IO_MODE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case GSC_IO_MODE_PIO:

			ptr	= "PIO";
			break;

		case GSC_IO_MODE_BMDMA:

			ptr	= "Block Mode DMA";
			break;

		case GSC_IO_MODE_DMDMA:

			errs	= 1;
			ptr		= "DMDMA: not supported";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


